@@ -115,6 +115,18 @@ try:
 except ImportError:
     log.debug('privex.helpers __init__ failed to import "converter", cannot load any converter.* modules...')
 
+try:
+    from privex.helpers.geoip import *
+    
+    plugin.HAS_GEOIP = True
+except ImportError:
+    log.debug('privex.helpers __init__ failed to import "geoip", not loading GeoIP2 helpers')
+
+try:
+    from privex.helpers.thread import *
+except ImportError:
+    log.debug('privex.helpers __init__ failed to import "thread", not loading python threading helpers')
+
 
 def _setup_logging(level=logging.WARNING):
     """
@@ -136,7 +148,7 @@ def _setup_logging(level=logging.WARNING):
 log = _setup_logging()
 name = 'helpers'
 
-VERSION = '2.11.0'
+VERSION = '2.14.0'
 
 
 
