@@ -29,6 +29,11 @@ class PrivexException(Exception):
     pass
 
 
+class NotFound(Exception):
+    """Generic exception mixin for all exceptions related to something not being found"""
+    pass
+
+
 #####
 # Exceptions related to DNS (domain, or individual record errors)
 #####
@@ -57,7 +62,7 @@ class BoundaryException(BaseDNSException):
     pass
 
 
-class DomainNotFound(BaseDNSException):
+class DomainNotFound(BaseDNSException, NotFound):
     """Thrown when a (sub)domain or it's parent(s) could not be found"""
     pass
 
@@ -67,7 +72,7 @@ class InvalidDNSRecord(BaseDNSException):
     pass
 
 
-class CacheNotFound(PrivexException):
+class CacheNotFound(PrivexException, NotFound):
     """
     Thrown when a cache key is requested, but it doesn't exist / is expired.
     
@@ -97,7 +102,7 @@ class EncryptionError(PrivexException):
     pass
 
 
-class EncryptKeyMissing(EncryptionError):
+class EncryptKeyMissing(EncryptionError, NotFound):
     """Raised when ENCRYPT_KEY is not set, or invalid"""
     pass
 
@@ -114,3 +119,47 @@ class SysCallError(PrivexException):
     (e.g. via :class:`subprocess.Popen`)
     """
 
+
+class GeoIPException(PrivexException):
+    pass
+
+
+class GeoIPDatabaseNotFound(GeoIPException, NotFound):
+    pass
+
+
+class GeoIPAddressNotFound(GeoIPException, NotFound):
+    pass
+
+
+class ReverseDNSNotFound(PrivexException, NotFound):
+    """Raised when a given IP address does not have a reverse DNS set"""
+    pass
+
+
+class InvalidHost(PrivexException, ValueError):
+    """Raised when a passed IP address or hostname/domain is invalid."""
+    pass
+
+
+class LockConflict(PrivexException):
+    """
+    Raised when attempting to acquire a lock with a :class:`threading.Lock` or :class:`asyncio.Lock`, and the lock object
+    is already locked.
+
+    This would only be raised either when non-blocking acquisition was requested, or the blocking wait timed out.
+    """
+
+
+class LockWaitTimeout(LockConflict):
+    """
+    Sub-class of :class:`.LockConflict` - only to be raised when a timeout has been reached while waiting
+    to acquire a :class:`threading.Lock`
+    """
+
+
+class EventWaitTimeout(PrivexException):
+    """
+    Raised when a timeout has been reached while waiting for an event (:class:`threading.Event`) to be signalled.
+    """
+
